# converter compensation example

clear all

close all



# define the plant transfer function
wp1 = 40e3;
k0 = 10;
T0 = tf([k0], [1/wp1^2, 0, 1]);

figure(1)
w = logspace(2, 6, 401);
bode(T0, w)
subplot(2, 1, 1)
title('uncompensated-bode')
print('uncompensated-bode', '-dpdf')
print('uncompensated-bode', '-dpng')

T0_poles = pole(T0)



# set the crossover frequency
k1 = 0.14674;
T1 = k1 * T0;

figure(2)
bode(T1, w)
subplot(2, 1, 1)
title('set-wc-bode')
print('set-wc-bode', '-dpdf')
print('set-wc-bode', '-dpng')

T1_poles = pole(T1)



# the first: lead compensation
wz = 21.74e3;
wp = 181.59e3;
k2 = k0 * k1 * sqrt(wz / wp);
T2 = T1 * sqrt(wz / wp) * tf([1/wz, 1], [1/wp, 1]);

figure(3)
bode(T2, w)
subplot(2, 1, 1)
title('lead-bode')
print('lead-bode', '-dpdf')
print('lead-bode', '-dpng')

S2 = minreal(T2/(1+T2));

z = abs(zero(S2));
p = abs(pole(S2));
pz = [p', z'];
m = 1.1 * max(pz);

figure(4)
pzmap(S2)
title('lead-pzmap')
axis([- m, m, - m, m])
axis('square')
print('lead-pzmap', '-dpdf')
print('lead-pzmap', '-dpng')

tend = 2.5e-3;

figure(5)
step(S2, tend)
title('lead-step')
print('lead-step', '-dpdf')
print('lead-step', '-dpng')

S2poles = pole(S2)



# +lag compensation, the first case

wl = 6283.2;
w0 = wl * k2;
T3 = tf(w0 * [1/wl, 1], [1, 0]) * tf([1/wz, 1], [1/wp, 1]) * tf([1], [1/wp1^2, 0, 1]);

figure(6)
bode(T3, w)
subplot(2, 1, 1)
title('lead-lag-bode')
print('lead-lag-bode', '-dpdf')
print('lead-lag-bode', '-dpng')

S3 = minreal(T3 / (1 + T3));

z = abs(zero(S3));
p = abs(pole(S3));
pz = [p', z'];
m = 1.1 * max(pz);

figure(7)
pzmap(S3);
title('lead-lag-pzmap')
axis([- m, m, - m, m])
axis('square')
print('lead-lag-pzmap', '-dpdf')
print('lead-lag-pzmap', '-dpng')

figure(8)
step(S3, tend)
title('lead-lag-step')
print('lead-lag-step', '-dpdf')
print('lead-lag-step', '-dpng')

S3poles = pole(S3)



# +lag compensation, the second case

wz = 18.017e3;
wp = 219.12e3;
k2 = 10 * k1 * sqrt(wz / wp);
wl = 6283.2;
w0 = wl * k2;

T4 = tf(w0 * [1/wl, 1], [1, 0]) * tf([1/wz, 1], [1/wp, 1]) * tf([1], [1/wp1^2, 0, 1]);

figure(9)
bode(T4, w)
subplot(2, 1, 1)
title('lead-lag-2-bode');
print('lead-lag-2-bode', '-dpdf')
print('lead-lag-2-bode', '-dpng')

S4 = minreal(T4 / (1 + T4));

z = abs(zero(S4));
p = abs(pole(S4));
pz = [p', z'];
m = 1.1 * max(pz);

figure(10)
pzmap(S4)
title('lead-lag-2-pzmap')
axis([- m, m, - m, m])
axis('square')
print('lead-lag-2-pzmap', '-dpdf')
print('lead-lag-2-pzmap', '-dpng')

figure(11)
step(S4, tend)
title('lead-lag-2-step')
print('lead-lag-2-step', '-dpdf')
print('lead-lag-2-step', '-dpng')

S4poles = pole(S4)



# optimal
wc = 10e3 * 2 * pi;
wz = wc / 10;
wp = 10 * wc;
k2 = k0 * k1 / (10 * sqrt(2));

T5 = tf(k2 * wc * [1/wc, 1], [1, 0]) * tf([1/wz, 1], [1/wp, 1]) * tf([1], [1/wp1^2, 0, 1]);

figure(12)
bode(T5, w)
subplot(2, 1, 1)
title('optimal-bode')
print('optimal-bode', '-dpdf')
print('optimal-bode', '-dpng')

S5 = minreal(T5 / (1 + T5));

z = abs(zero(S5));
p = abs(pole(S5));
pz = [p', z'];
m = 1.1 * max(pz);

figure(13)
pzmap(S5)
title('optimal-pzmap')
axis([- m, m, - m, m])
axis('square')
print('optimal-pzmap', '-dpdf')
print('optimal-pzmap', '-dpng')


figure(14);
step(S5, tend)
title('optimal-step')
print('optimal-step', '-dpdf')
print('optimal-step', '-dpng')

S5poles = pole(S5)



figure(15)
step(S2, S3, S4, S5, tend)
title('step-comparison')
print('step-comparison', '-dpdf')
print('step-comparison', '-dpng')



# closed loop system Bode plots

w = logspace(1, 5, 501);

figure(16)
bode(S2, w)
subplot(2, 1, 1)
title('closed-bode-lead')
print('closed-bode-lead', '-dpdf')
print('closed-bode-lead', '-dpng')

figure(17)
bode(S3, w)
subplot(2, 1, 1)
title('closed-bode-lead-lag')
print('closed-bode-lead-lag', '-dpdf')
print('closed-bode-lead-lag', '-dpng')

figure(18)
bode(S4, w)
subplot(2, 1, 1)
title('closed-bode-lead-lag-2')
print('closed-bode-lead-lag-2', '-dpdf')
print('closed-bode-lead-lag-2', '-dpng')

figure(19)
bode(S5, w)
subplot(2, 1, 1)
title('closed-bode-optimal')
print('closed-bode-optimal', '-dpdf')
print('closed-bode-optimal', '-dpng')

figure(20)
bode(S2, S3, S4, S5, w)
subplot(2, 1, 1)
title('closed-bode-comparison')
print('closed-bode-comparison', '-dpdf')
print('closed-bode-comparison', '-dpng')

